@@ -165,6 +165,13 @@ if [ "$DEPCHECK" == 1 ] ; then
 fi
 
 echo "========== Starting docker container =========="
+mkdir -p /var/local/run/
+if ! grep -q "unix:///var/local/run/docker.sock" /etc/default/docker; then
+cat << EOF >> /etc/default/docker
+DOCKER_OPTS="-H unix:///var/local/run/docker.sock"
+EOF
+fi
+export DOCKER_HOST="unix:///var/local/run/docker.sock"
 service docker start || true
 
 echo "========== Running faucet system tests =========="
@@ -174,6 +181,8 @@ export http_proxy=
 
 if [ "$INTEGRATIONTESTS" == 1 ] ; then
   echo "========== Running faucet integration tests =========="
+  sudo apt-get update -y
+  sudo apt-get install dnsmasq -y
   cd /faucet-src/tests/integration
   ./mininet_main.py -c
 elif [ "$GEN_INT" == 1 ] ; then
