@@ -17,6 +17,7 @@
 # limitations under the License.
 
 from collections import defaultdict
+from collections.abc import Iterable
 import copy
 import random
 import math
@@ -320,7 +321,7 @@ configuration.
         self.has_externals = None
         self.tunnel_acls = []
 
-        super(DP, self).__init__(_id, dp_id, conf)
+        super().__init__(_id, dp_id, conf)
 
     def __str__(self):
         return self.name
@@ -340,7 +341,7 @@ configuration.
 
     def check_config(self):
         """Check configuration of this dp"""
-        super(DP, self).check_config()
+        super().check_config()
         test_config_condition(not isinstance(self.dp_id, int), (
             'dp_id must be %s not %s' % (int, type(self.dp_id))))
         test_config_condition(self.dp_id < 0 or self.dp_id > 2**64-1, (
@@ -577,7 +578,7 @@ configuration.
         self.tables = tables
 
     def set_defaults(self):
-        super(DP, self).set_defaults()
+        super().set_defaults()
         self._set_default('dp_id', self._id)
         self._set_default('name', str(self._id))
         self._set_default('lowest_priority', self.priority_offset)
@@ -773,18 +774,31 @@ configuration.
 
     def reset_refs(self, vlans=None):
         """Resets VLAN references."""
+
         if vlans is None:
             vlans = self.vlans
-            router_vlans = [vlan._id for router in self.routers.values() for vlan in router.vlans]
+            router_vlans = {vlan._id for router in self.routers.values() for vlan in router.vlans}
         else:
-            router_vlans = [vlan for router in self.routers.values() for vlan in router.vlans]
+            router_vlans = {vlan for router in self.routers.values() for vlan in router.vlans}
+
+        vlan_ports = defaultdict(set)
+        for port in self.ports.values():
+            for vlan in port.vlans():
+                vlan_ports[vlan].add(port)
+
+        if self.stack_ports or self.stack.is_root():
+            new_vlans = list(vlans.values())
+        else:
+            new_vlans = []
+            for vlan in vlans.values():
+                if (vlan_ports[vlan] or vlan.reserved_internal_vlan or
+                        vlan.dot1x_assigned or vlan._id in router_vlans):
+                    new_vlans.append(vlan)
+
         self.vlans = {}
-        for vlan in vlans.values():
-            vlan.reset_ports(self.ports.values())
-            if (vlan.get_ports() or vlan.reserved_internal_vlan or
-                    vlan.dot1x_assigned or vlan._id in router_vlans or
-                    self.stack_ports() or self.stack.is_root()):
-                self.vlans[vlan.vid] = vlan
+        for vlan in new_vlans:
+            vlan.reset_ports(vlan_ports[vlan])
+            self.vlans[vlan.vid] = vlan
 
     def resolve_port(self, port_name):
         """Resolve a port by number or name."""
@@ -1119,7 +1133,7 @@ configuration.
             self._lldp_defaults()
 
         test_config_condition(
-            not self.vlans and not self.stack_ports(),
+            not self.vlans and not self.non_vlan_ports(),
             'no VLANs referenced by interfaces in %s' % self.name)
         dp_by_name = {dp.name: dp for dp in dps}
         vlan_by_name = {vlan.name: vlan for vlan in self.vlans.values()}
@@ -1150,7 +1164,7 @@ configuration.
     def finalize(self):
         """Need to configure OF tables as very last step."""
         self._configure_tables()
-        super(DP, self).finalize()
+        super().finalize()
 
     def get_native_vlan(self, port_num):
         """Return native VLAN for a port by number, or None."""
@@ -1303,34 +1317,33 @@ configuration.
 
         # Detect changes to VLANs and ACLs based on port changes.
         if not all_ports_changed:
+            def get_vids(vlans):
+                if not vlans:
+                    return set()
+                if isinstance(vlans, Iterable):
+                    return {vlan.vid for vlan in vlans}
+                return {vlans.vid}
+
             def _add_changed_vlan_port(port, port_dp):
-                if port.native_vlan:
-                    changed_vlans.add(port.native_vlan.vid)
-                if port.tagged_vlans:
-                    changed_vlans.update({vlan.vid for vlan in port.tagged_vlans})
+                changed_vlans.update(get_vids(port.vlans()))
                 if port.stack:
-                    changed_vlans.update({vlan.vid for vlan in port_dp.vlans.values()})
+                    changed_vlans.update(get_vids(port_dp.vlans.values()))
+
+            def _add_changed_vlans(old_port, new_port):
+                if old_port.vlans() != new_port.vlans():
+                    old_vids = get_vids(old_port.vlans())
+                    new_vids = get_vids(new_port.vlans())
+                    changed_vlans.update(old_vids.symmetric_difference(new_vids))
+                # stacking dis/enabled on a port.
+                if bool(old_port.stack) != bool(new_port.stack):
+                    changed_vlans.update(get_vids(new_dp.vlans.values()))
 
             for port_no in changed_ports:
                 if port_no not in self.ports:
                     continue
                 old_port = self.ports[port_no]
                 new_port = new_dp.ports[port_no]
-                # native_vlan changed.
-                if old_port.native_vlan != new_port.native_vlan:
-                    for port in (old_port, new_port):
-                        if port.native_vlan:
-                            changed_vlans.add(port.native_vlan.vid)
-                # tagged vlans changed.
-                if old_port.tagged_vlans != new_port.tagged_vlans:
-                    for port in (old_port, new_port):
-                        if port.tagged_vlans:
-                            changed_vlans.update({vlan.vid for vlan in port.tagged_vlans})
-                # stacking dis/enabled on a port.
-                if bool(old_port.stack) != bool(new_port.stack):
-                    changed_vlans.update({vlan.vid for vlan in new_dp.vlans.values()})
-
-            # ports deleted or added.
+                _add_changed_vlans(old_port, new_port)
             for port_no in deleted_ports:
                 port = self.ports[port_no]
                 _add_changed_vlan_port(port, self)
@@ -1340,14 +1353,10 @@ configuration.
             for port_no in same_ports:
                 old_port = self.ports[port_no]
                 new_port = new_dp.ports[port_no]
-                # mirror options changed.
                 if old_port.mirror != new_port.mirror:
                     logger.info('port %s mirror options changed: %s' % (
                         port_no, new_port.mirror))
                     changed_ports.add(port_no)
-                    for mirrored_port, port_dp in ((old_port, self), (new_port, new_dp)):
-                        if mirrored_port.mirror:
-                            _add_changed_vlan_port(mirrored_port, port_dp)
                 # ACL changes
                 new_acl_ids = new_port.acls_in
                 port_acls_changed = set()
@@ -1361,7 +1370,7 @@ configuration.
                     changed_acl_ports.add(port_no)
                     logger.info('port %s ACL changed (ACL %s content changed)' % (
                         port_no, port_acls_changed))
-                elif old_acl_ids != new_acl_ids:
+                elif (old_acl_ids or new_acl_ids) and old_acl_ids != new_acl_ids:
                     changed_acl_ports.add(port_no)
                     logger.info('port %s ACL changed (ACL %s to %s)' % (
                         port_no, old_acl_ids, new_acl_ids))
@@ -1393,7 +1402,8 @@ configuration.
             all_ports_changed = True
 
         return (all_ports_changed, deleted_ports,
-                changed_ports, added_ports, changed_acl_ports, changed_vlans)
+                changed_ports, added_ports, changed_acl_ports,
+                changed_vlans)
 
     def _get_meter_config_changes(self, logger, new_dp):
         """Detect any config changes to meters.
