@@ -188,7 +188,7 @@ def ignore_port(port_num):
         bool: True if FAUCET should ignore this port.
     """
     # special case OFPP_LOCAL to allow FAUCET to manage switch admin interface.
-    if port_num == ofp.OFPP_LOCAL:
+    if port_num in (ofp.OFPP_LOCAL, ofp.OFPP_IN_PORT):
         return False
     # 0xF0000000 and up are not physical ports.
     return port_num > 0xF0000000
