@@ -194,6 +194,54 @@ dps:
         self.update_and_revert_config(self.CONFIG, self.LESS_CONFIG, 'cold')
 
 
+class ValveAddPortMirrorNoDelVLANTestCase(ValveTestBases.ValveTestNetwork):
+    """Test addition of port mirroring does not cause a del VLAN."""
+
+    CONFIG = """
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 1
+                tagged_vlans: [0x100]
+            p2:
+                number: 2
+                tagged_vlans: [0x100]
+            p3:
+                number: 3
+                output_only: true
+""" % DP1_CONFIG
+
+    MORE_CONFIG = """
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 1
+                tagged_vlans: [0x100]
+            p2:
+                number: 2
+                tagged_vlans: [0x100]
+            p3:
+                number: 3
+                output_only: true
+                mirror: [1]
+""" % DP1_CONFIG
+
+    def setUp(self):
+        self.setup_valves(self.CONFIG)[self.DP_ID]
+
+    def test_port_mirror(self):
+        """Test addition of port mirroring does not cause a del VLAN."""
+        reload_ofmsgs = self.update_config(self.MORE_CONFIG, reload_type='warm')[self.DP_ID]
+        for ofmsg in reload_ofmsgs:
+            if valve_of.is_flowdel(ofmsg):
+                if ofmsg.table_id == valve_of.ofp.OFPTT_ALL and ofmsg.match:
+                    self.assertNotIn('vlan_vid', ofmsg.match, ofmsg)
+
+
 class ValveAddPortTestCase(ValveTestBases.ValveTestNetwork):
     """Test addition of a port."""
 
@@ -226,7 +274,8 @@ dps:
                 tagged_vlans: [0x100]
 """ % DP1_CONFIG
 
-    def _inport_flows(self, in_port, ofmsgs):
+    @staticmethod
+    def _inport_flows(in_port, ofmsgs):
         return [
             ofmsg for ofmsg in ValveTestBases.flowmods_from_flows(ofmsgs)
             if ofmsg.match.get('in_port') == in_port]
@@ -872,6 +921,7 @@ dps:
 
 
 class ValveTestVLANRef(ValveTestBases.ValveTestNetwork):
+    """Test reference to same VLAN by name or VID."""
 
     CONFIG = """
 dps:
@@ -893,6 +943,7 @@ vlans:
         self.setup_valves(self.CONFIG)
 
     def test_vlan_refs(self):
+        """Test same VLAN is referred to."""
         vlans = self.valves_manager.valves[self.DP_ID].dp.vlans
         self.assertEqual(1, len(vlans))
         self.assertEqual('threes', vlans[333].name, vlans[333])
@@ -1110,7 +1161,7 @@ class ValveReloadConfigTestCase(ValveTestBases.ValveTestBig):
     """Repeats the tests after a config reload."""
 
     def setUp(self):
-        super(ValveReloadConfigTestCase, self).setUp()
+        super().setUp()
         self.flap_port(1)
         self.update_config(CONFIG, reload_type='warm', reload_expected=False)
 
